@@ -1,48 +1,37 @@
-#ifndef __NOTIFICATION_H__
-#define __NOTIFICATION_H__
+#ifndef FILEZILLA_ENGINE_NOTIFICATION_HEADER
+#define FILEZILLA_ENGINE_NOTIFICATION_HEADER
 
 // Notification overview
 // ---------------------
 
 // To inform the application about what's happening, the engine sends
-// some notifications to the application using events.
-// For example the event table entry could look like this:
-//     EVT_FZ_NOTIFICATION(wxID_ANY, CMainFrame::OnEngineEvent)
-// and the handler function has the following declaration:
-//     void OnEngineEvent(wxFzEvent& event);
-// You can get the engine which sent the event by reading
-//     event.engine_
+// some notifications to the application.
+// The handler needs to derive from EngineNotificationHandler and implement
+// the OnEngineEvent method which takes the engine as parameter.
 // Whenever you get a notification event,
 // CFileZillaEngine::GetNextNotification has to be called until it returns 0,
 // or you will lose important notifications or your memory will fill with
 // pending notifications.
+//
+// Note: It may be called from a worker thread.
 
 // A special class of notifications are the asynchronous requests. These
-// requests have to be answered. Once proessed, call
+// requests have to be answered. Once processed, call
 // CFileZillaEngine::SetAsyncRequestReply to continue the current operation.
 
 #include "local_path.h"
-#include "timeex.h"
+#include <libfilezilla/time.hpp>
 
 class CFileZillaEngine;
-class wxFzEvent final : public wxEvent
+
+class EngineNotificationHandler
 {
 public:
-	wxFzEvent();
+	virtual ~EngineNotificationHandler() {}
 
-	explicit wxFzEvent(int id);
-	explicit wxFzEvent(CFileZillaEngine const* engine);
-
-	virtual wxEvent *Clone() const;
-
-	CFileZillaEngine const* engine_;
+	virtual void OnEngineEvent(CFileZillaEngine* engine) = 0;
 };
 
-wxDECLARE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
-
-#define EVT_FZ_NOTIFICATION(id, func) \
-	wx__DECLARE_EVT1(fzEVT_NOTIFICATION, id, &func)
-
 enum NotificationId
 {
 	nId_logmsg,				// notification about new messages for the message log
@@ -71,7 +60,7 @@ enum RequestId
 class CNotification
 {
 public:
-	virtual ~CNotification() {}; // TODO: One GCC >= 4.8 is in Debian Stable (Jessie by then), make default and add testcase to configure.
+	virtual ~CNotification() = default;
 	virtual NotificationId GetID() const = 0;
 
 protected:
@@ -106,7 +95,7 @@ public:
 	{
 	}
 
-	wxString msg;
+	std::wstring msg;
 	MessageType msgType{MessageType::Status}; // Type of message, see logging.h for details
 };
 
@@ -140,7 +129,7 @@ protected:
 class CAsyncRequestNotification : public CNotificationHelper<nId_asyncrequest>
 {
 public:
-	virtual enum RequestId GetRequestID() const = 0;
+	virtual RequestId GetRequestID() const = 0;
 	unsigned int requestNumber{}; // Do never change this
 
 protected:
@@ -152,18 +141,18 @@ protected:
 class CFileExistsNotification final : public CAsyncRequestNotification
 {
 public:
-	virtual enum RequestId GetRequestID() const;
+	virtual RequestId GetRequestID() const;
 
 	bool download{};
 
-	wxString localFile;
+	std::wstring localFile;
 	int64_t localSize{-1};
-	CDateTime localTime;
+	fz::datetime localTime;
 
-	wxString remoteFile;
+	std::wstring remoteFile;
 	CServerPath remotePath;
 	int64_t remoteSize{-1};
-	CDateTime remoteTime;
+	fz::datetime remoteTime;
 
 	bool ascii{};
 
@@ -190,7 +179,7 @@ public:
 
 	// Set to new filename if overwriteAction is rename. Might trigger further
 	// file exists notifications if new target file exists as well.
-	wxString newName;
+	std::wstring newName;
 };
 
 class CInteractiveLoginNotification final : public CAsyncRequestNotification
@@ -201,8 +190,8 @@ public:
 		keyfile
 	};
 
-	CInteractiveLoginNotification(type t, const wxString& challenge, bool repeated);
-	virtual enum RequestId GetRequestID() const;
+	CInteractiveLoginNotification(type t, std::wstring const& challenge, bool repeated);
+	virtual RequestId GetRequestID() const;
 
 	// Set to true if you have set a password
 	bool passwordSet{};
@@ -210,7 +199,7 @@ public:
 	// Set password by calling server.SetUser
 	CServer server;
 
-	const wxString& GetChallenge() const { return m_challenge; }
+	std::wstring const& GetChallenge() const { return m_challenge; }
 
 	type GetType() const { return m_type; }
 
@@ -218,7 +207,7 @@ public:
 
 protected:
 	// Password prompt string as given by the server
-	wxString const m_challenge;
+	std::wstring const m_challenge;
 
 	type const m_type;
 
@@ -240,17 +229,17 @@ class CTransferStatus final
 {
 public:
 	CTransferStatus() {}
-	CTransferStatus(wxFileOffset total, wxFileOffset start, bool l)
+	CTransferStatus(int64_t total, int64_t start, bool l)
 		: totalSize(total)
 		, startOffset(start)
 		, currentOffset(start)
 		, list(l)
 	{}
 
-	CDateTime started;
-	wxFileOffset totalSize{-1};		// Total size of the file to transfer, -1 if unknown
-	wxFileOffset startOffset{-1};
-	wxFileOffset currentOffset{-1};
+	fz::datetime started;
+	int64_t totalSize{-1};		// Total size of the file to transfer, -1 if unknown
+	int64_t startOffset{-1};
+	int64_t currentOffset{-1};
 
 	void clear() { startOffset = -1; }
 	bool empty() const { return startOffset < 0; }
@@ -259,7 +248,7 @@ public:
 
 	// True on download notifications iff currentOffset != startOffset.
 	// True on FTP upload notifications iff currentOffset != startOffset
-	// AND after the first accepted data after the first wxSOCKET_WOULDBLOCK.
+	// AND after the first accepted data after the first EWOULDBLOCK.
 	// SFTP uploads: Set to true if currentOffset >= startOffset + 65536.
 	bool madeProgress{};
 
@@ -278,17 +267,34 @@ protected:
 	CTransferStatus const status_;
 };
 
-// Notification about new or changed hostkeys, only used by SSH/SFTP transfers.
-// GetRequestID() returns either reqId_hostkey or reqId_hostkeyChanged
-class CHostKeyNotification final : public CAsyncRequestNotification
+class CSftpEncryptionDetails
 {
 public:
-	CHostKeyNotification(wxString host, int port, wxString fingerprint, bool changed = false);
-	virtual enum RequestId GetRequestID() const;
+	virtual ~CSftpEncryptionDetails() = default;
 
-	wxString GetHost() const;
+	std::wstring hostKeyAlgorithm;
+	std::wstring hostKeyFingerprintMD5;
+	std::wstring hostKeyFingerprintSHA256;
+	std::wstring kexAlgorithm;
+	std::wstring kexHash;
+	std::wstring kexCurve;
+	std::wstring cipherClientToServer;
+	std::wstring cipherServerToClient;
+	std::wstring macClientToServer;
+	std::wstring macServerToClient;
+};
+
+// Notification about new or changed hostkeys, only used by SSH/SFTP transfers.
+// GetRequestID() returns either reqId_hostkey or reqId_hostkeyChanged
+class CHostKeyNotification final : public CAsyncRequestNotification, public CSftpEncryptionDetails
+{
+public:
+	CHostKeyNotification(std::wstring const& host, int port, CSftpEncryptionDetails const& details, bool changed = false);
+
+	virtual RequestId GetRequestID() const;
+
+	std::wstring GetHost() const;
 	int GetPort() const;
-	wxString GetFingerprint() const;
 
 	// Set to true if you trust the server
 	bool m_trust{};
@@ -299,9 +305,8 @@ public:
 
 protected:
 
-	const wxString m_host;
+	const std::wstring m_host;
 	const int m_port;
-	const wxString m_fingerprint;
 	const bool m_changed;
 };
 
@@ -322,87 +327,96 @@ class CCertificate final
 {
 public:
 	CCertificate() = default;
+	CCertificate(CCertificate const& op) = default;
+
 	CCertificate(
-		unsigned char const* rawData, unsigned int len,
-		CDateTime const& activationTime, CDateTime const& expirationTime,
-		wxString const& serial,
-		wxString const& pkalgoname, unsigned int bits,
-		wxString const& signalgoname,
-		wxString const& fingerprint_sha256,
-		wxString const& fingerprint_sha1,
-		wxString const& issuer,
-		wxString const& subject,
-		std::vector<wxString> const& altSubjectNames);
+		std::vector<uint8_t> const& rawData,
+		fz::datetime const& activationTime, fz::datetime const& expirationTime,
+		std::wstring const& serial,
+		std::wstring const& pkalgoname, unsigned int bits,
+		std::wstring const& signalgoname,
+		std::wstring const& fingerprint_sha256,
+		std::wstring const& fingerprint_sha1,
+		std::wstring const& issuer,
+		std::wstring const& subject,
+		std::vector<std::wstring> const& altSubjectNames);
 
-	CCertificate(CCertificate const& op);
-	~CCertificate();
+	CCertificate(
+		std::vector<uint8_t> && rawdata,
+		fz::datetime const& activationTime, fz::datetime const& expirationTime,
+		std::wstring const& serial,
+		std::wstring const& pkalgoname, unsigned int bits,
+		std::wstring const& signalgoname,
+		std::wstring const& fingerprint_sha256,
+		std::wstring const& fingerprint_sha1,
+		std::wstring const& issuer,
+		std::wstring const& subject,
+		std::vector<std::wstring> && altSubjectNames);
 
-	const unsigned char* GetRawData(unsigned int& len) const { len = m_len; return m_rawData; }
-	CDateTime GetActivationTime() const { return m_activationTime; }
-	CDateTime GetExpirationTime() const { return m_expirationTime; }
 
-	const wxString& GetSerial() const { return m_serial; }
-	const wxString& GetPkAlgoName() const { return m_pkalgoname; }
+	std::vector<uint8_t> GetRawData() const { return m_rawData; }
+	fz::datetime GetActivationTime() const { return m_activationTime; }
+	fz::datetime GetExpirationTime() const { return m_expirationTime; }
+
+	std::wstring const& GetSerial() const { return m_serial; }
+	std::wstring const& GetPkAlgoName() const { return m_pkalgoname; }
 	unsigned int GetPkAlgoBits() const { return m_pkalgobits; }
 
-	const wxString& GetSignatureAlgorithm() const { return m_signalgoname; }
+	std::wstring const& GetSignatureAlgorithm() const { return m_signalgoname; }
 
-	const wxString& GetFingerPrintSHA256() const { return m_fingerprint_sha256; }
-	const wxString& GetFingerPrintSHA1() const { return m_fingerprint_sha1; }
+	std::wstring const& GetFingerPrintSHA256() const { return m_fingerprint_sha256; }
+	std::wstring const& GetFingerPrintSHA1() const { return m_fingerprint_sha1; }
 
-	const wxString& GetSubject() const { return m_subject; }
-	const wxString& GetIssuer() const { return m_issuer; }
+	std::wstring const& GetSubject() const { return m_subject; }
+	std::wstring const& GetIssuer() const { return m_issuer; }
 
-	CCertificate& operator=(CCertificate const& op);
-
-	std::vector<wxString> const& GetAltSubjectNames() const { return m_altSubjectNames; }
+	std::vector<std::wstring> const& GetAltSubjectNames() const { return m_altSubjectNames; }
 
 private:
-	CDateTime m_activationTime;
-	CDateTime m_expirationTime;
+	fz::datetime m_activationTime;
+	fz::datetime m_expirationTime;
 
-	unsigned char* m_rawData{};
-	unsigned int m_len{};
+	std::vector<uint8_t> m_rawData{};
 
-	wxString m_serial;
-	wxString m_pkalgoname;
+	std::wstring m_serial;
+	std::wstring m_pkalgoname;
 	unsigned int m_pkalgobits{};
 
-	wxString m_signalgoname;
+	std::wstring m_signalgoname;
 
-	wxString m_fingerprint_sha256;
-	wxString m_fingerprint_sha1;
+	std::wstring m_fingerprint_sha256;
+	std::wstring m_fingerprint_sha1;
 
-	wxString m_issuer;
-	wxString m_subject;
+	std::wstring m_issuer;
+	std::wstring m_subject;
 
-	std::vector<wxString> m_altSubjectNames;
+	std::vector<std::wstring> m_altSubjectNames;
 };
 
 class CCertificateNotification final : public CAsyncRequestNotification
 {
 public:
-	CCertificateNotification(const wxString& host, unsigned int port,
-		const wxString& protocol,
-		const wxString& keyExchange,
-		const wxString& sessionCipher,
-		const wxString& sessionMac,
+	CCertificateNotification(std::wstring const& host, unsigned int port,
+		std::wstring const& protocol,
+		std::wstring const& keyExchange,
+		std::wstring const& sessionCipher,
+		std::wstring const& sessionMac,
 		int algorithmWarnings,
-		const std::vector<CCertificate> &certificates);
-	virtual enum RequestId GetRequestID() const { return reqId_certificate; }
+		std::vector<CCertificate> && certificates);
+	virtual RequestId GetRequestID() const { return reqId_certificate; }
 
-	const wxString& GetHost() const { return m_host; }
+	std::wstring const& GetHost() const { return m_host; }
 	unsigned int GetPort() const { return m_port; }
 
-	const wxString& GetSessionCipher() const { return m_sessionCipher; }
-	const wxString& GetSessionMac() const { return m_sessionMac; }
+	std::wstring const& GetSessionCipher() const { return m_sessionCipher; }
+	std::wstring const& GetSessionMac() const { return m_sessionMac; }
 
 	bool m_trusted{};
 
 	const std::vector<CCertificate> GetCertificates() const { return m_certificates; }
 
-	const wxString& GetProtocol() const { return m_protocol; }
-	const wxString& GetKeyExchange() const { return m_keyExchange; }
+	std::wstring const& GetProtocol() const { return m_protocol; }
+	std::wstring const& GetKeyExchange() const { return m_keyExchange; }
 
 	enum algorithm_warnings_t
 	{
@@ -415,32 +429,23 @@ public:
 	int GetAlgorithmWarnings() const { return m_algorithmWarnings; }
 
 private:
-	wxString m_host;
+	std::wstring m_host;
 	unsigned int m_port{};
 
-	wxString m_protocol;
-	wxString m_keyExchange;
-	wxString m_sessionCipher;
-	wxString m_sessionMac;
+	std::wstring m_protocol;
+	std::wstring m_keyExchange;
+	std::wstring m_sessionCipher;
+	std::wstring m_sessionMac;
 	int m_algorithmWarnings{};
 
 	std::vector<CCertificate> m_certificates;
 };
 
-class CSftpEncryptionNotification : public CNotificationHelper<nId_sftp_encryption>
+class CSftpEncryptionNotification final : public CNotificationHelper<nId_sftp_encryption>, public CSftpEncryptionDetails
 {
-public:
-	wxString hostKey;
-	wxString kexAlgorithm;
-	wxString kexHash;
-	wxString kexCurve;
-	wxString cipherClientToServer;
-	wxString cipherServerToClient;
-	wxString macClientToServer;
-	wxString macServerToClient;
 };
 
-class CLocalDirCreatedNotification : public CNotificationHelper<nId_local_dir_created>
+class CLocalDirCreatedNotification final : public CNotificationHelper<nId_local_dir_created>
 {
 public:
 	CLocalPath dir;
