@@ -1,15 +1,15 @@
 #include <filezilla.h>
 #include "Options.h"
 #include "filezillaapp.h"
-#include <wx/tokenzr.h>
 #include "ipcmutex.h"
+#include "locale_initializer.h"
 #include <option_change_event_handler.h>
 #include "sizeformatting.h"
 
 #include <algorithm>
 #include <string>
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	#include <shlobj.h>
 
 	// Needed for MinGW:
@@ -38,11 +38,11 @@ struct t_Option
 {
 	const char name[30];
 	const Type type;
-	const wxString defaultValue; // Default values are stored as string even for numerical options
+	const std::wstring defaultValue; // Default values are stored as string even for numerical options
 	const Flags flags; // internal items won't get written to settings file nor loaded from there
 };
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 //case insensitive
 #define DEFAULT_FILENAME_SORT   _T("0")
 #else
@@ -79,8 +79,8 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Reconnect count", number, _T("2"), normal },
 	{ "Reconnect delay", number, _T("5"), normal },
 	{ "Enable speed limits", number, _T("0"), normal },
-	{ "Speedlimit inbound", number, _T("100"), normal },
-	{ "Speedlimit outbound", number, _T("20"), normal },
+	{ "Speedlimit inbound", number, _T("1000"), normal },
+	{ "Speedlimit outbound", number, _T("100"), normal },
 	{ "Speedlimit burst tolerance", number, _T("0"), normal },
 	{ "Preallocate space", number, _T("0"), normal },
 	{ "View hidden files", number, _T("0"), normal },
@@ -107,6 +107,8 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Size format", number, _T("0"), normal },
 	{ "Size thousands separator", number, _T("1"), normal },
 	{ "Size decimal places", number, _T("1"), normal },
+	{ "TCP Keepalive Interval", number, _T("15"), normal },
+	{ "Cache TTL", number, _T("600"), normal },
 
 	// Interface settings
 	{ "Number of Transfers", number, _T("2"), normal },
@@ -114,7 +116,6 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|sha1|sha256|sha512|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
 	{ "Auto Ascii no extension", number, _T("1"), normal },
 	{ "Auto Ascii dotfiles", number, _T("1"), normal },
-	{ "Theme", string, _T("opencrystal/"), normal },
 	{ "Language Code", string, _T(""), normal },
 	{ "Last Server Path", string, _T(""), normal },
 	{ "Concurrent download limit", number, _T("0"), normal },
@@ -122,6 +123,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Update Check", number, _T("1"), normal },
 	{ "Update Check Interval", number, _T("7"), normal },
 	{ "Last automatic update check", string, _T(""), normal },
+	{ "Last automatic update version", string, _T(""), normal },
 	{ "Update Check New Version", string, _T(""), normal },
 	{ "Update Check Check Beta", number, _T("0"), normal },
 	{ "Show debug menu", number, _T("0"), normal },
@@ -129,6 +131,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "File exists action upload", number, _T("0"), normal },
 	{ "Allow ascii resume", number, _T("0"), normal },
 	{ "Greeting version", string, _T(""), normal },
+	{ "Greeting resources", string, _T(""), normal },
 	{ "Onetime Dialogs", string, _T(""), normal },
 	{ "Show Tree Local", number, _T("1"), normal },
 	{ "Show Tree Remote", number, _T("1"), normal },
@@ -156,7 +159,8 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Comparison mode", number, _T("1"), normal },
 	{ "Comparison threshold", number, _T("1"), normal },
 	{ "Site Manager position", string, _T(""), normal },
-	{ "Theme icon size", string, _T(""), normal },
+	{ "Icon theme", string, _T("default"), normal },
+	{ "Icon scale", number, _T("125"), normal },
 	{ "Timestamp in message log", number, _T("0"), normal },
 	{ "Sitemanager last selected", string, _T(""), normal },
 	{ "Local filelist shown columns", string, _T(""), normal },
@@ -190,34 +194,33 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Show Site Manager on startup", number, _T("0"), normal },
 	{ "Prompt password change", number, _T("0"), normal },
 	{ "Persistent Choices", number, _T("0"), normal },
+	{ "Queue completion action", number, _T("1"), normal },
+	{ "Queue completion command", string, _T(""), normal },
+	{ "Drag and Drop disabled", number, _T("0"), normal },
+	{ "Disable update footer", number, _T("0"), normal },
 
 	// Default/internal options
 	{ "Config Location", string, _T(""), default_only },
 	{ "Kiosk mode", number, _T("0"), default_priority },
-	{ "Disable update check", number, _T("0"), default_only }
+	{ "Disable update check", number, _T("0"), default_only },
+	{ "Cache directory", string, _T(""), default_priority },
 };
 
 BEGIN_EVENT_TABLE(COptions, wxEvtHandler)
 EVT_TIMER(wxID_ANY, COptions::OnTimer)
 END_EVENT_TABLE()
 
-t_OptionsCache& t_OptionsCache::operator=(wxString const& v)
+t_OptionsCache& t_OptionsCache::operator=(std::wstring const& v)
 {
 	strValue = v;
-	long l;
-	if (v.ToLong(&l)) {
-		numValue = l;
-	}
-	else {
-		numValue = 0;
-	}
+	numValue = fz::to_integral<int>(v);
 	return *this;
 }
 
 t_OptionsCache& t_OptionsCache::operator=(int v)
 {
 	numValue = v;
-	strValue.Printf("%d", v);
+	strValue = fz::to_wstring(v);
 	return *this;
 }
 
@@ -244,8 +247,9 @@ COptions::COptions()
 		delete m_pXmlFile;
 		m_pXmlFile = 0;
 	}
-	else
+	else {
 		CreateSettingsXmlElement();
+	}
 
 	LoadOptions(nameOptionMap);
 }
@@ -254,8 +258,9 @@ std::map<std::string, unsigned int> COptions::GetNameOptionMap() const
 {
 	std::map<std::string, unsigned int> ret;
 	for (unsigned int i = 0; i < OPTIONS_NUM; ++i) {
-		if (options[i].flags != internal)
+		if (options[i].flags != internal) {
 			ret.insert(std::make_pair(std::string(options[i].name), i));
+		}
 	}
 	return ret;
 }
@@ -270,45 +275,46 @@ COptions::~COptions()
 
 int COptions::GetOptionVal(unsigned int nID)
 {
-	if (nID >= OPTIONS_NUM)
+	if (nID >= OPTIONS_NUM) {
 		return 0;
+	}
 
-	scoped_lock l(m_sync_);
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].numValue;
 }
 
-wxString COptions::GetOption(unsigned int nID)
+std::wstring COptions::GetOption(unsigned int nID)
 {
-	if (nID >= OPTIONS_NUM)
-		return wxString();
+	if (nID >= OPTIONS_NUM) {
+		return std::wstring();
+	}
 
-	scoped_lock l(m_sync_);
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].strValue;
 }
 
 bool COptions::SetOption(unsigned int nID, int value)
 {
-	if (nID >= OPTIONS_NUM)
+	if (nID >= OPTIONS_NUM) {
 		return false;
+	}
 
-	if (options[nID].type != number)
+	if (options[nID].type != number) {
 		return false;
+	}
 
 	ContinueSetOption(nID, value);
 	return true;
 }
 
-bool COptions::SetOption(unsigned int nID, wxString const& value)
+bool COptions::SetOption(unsigned int nID, std::wstring const& value)
 {
-	if (nID >= OPTIONS_NUM)
+	if (nID >= OPTIONS_NUM) {
 		return false;
+	}
 
 	if (options[nID].type != string) {
-		long tmp;
-		if (!value.ToLong(&tmp))
-			return false;
-
-		return SetOption(nID, tmp);
+		return SetOption(nID, fz::to_integral<int>(value));
 	}
 
 	ContinueSetOption(nID, value);
@@ -321,7 +327,7 @@ void COptions::ContinueSetOption(unsigned int nID, T const& value)
 	T validated = Validate(nID, value);
 
 	{
-		scoped_lock l(m_sync_);
+		fz::scoped_lock l(m_sync_);
 		if (m_optionsCache[nID] == validated) {
 			// Nothing to do
 			return;
@@ -330,14 +336,16 @@ void COptions::ContinueSetOption(unsigned int nID, T const& value)
 	}
 
 	// Fixme: Setting options from other threads
-	if (!wxIsMainThread())
+	if (!wxIsMainThread()) {
 		return;
+	}
 
 	if (options[nID].flags == normal || options[nID].flags == default_priority) {
 		SetXmlValue(nID, validated);
 
-		if (!m_save_timer.IsRunning())
+		if (!m_save_timer.IsRunning()) {
 			m_save_timer.Start(15000, true);
+		}
 	}
 
 	if (changedOptions_.none()) {
@@ -356,17 +364,19 @@ void COptions::NotifyChangedOptions()
 
 bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 {
-	if (nID >= OPTIONS_NUM)
+	if (nID >= OPTIONS_NUM) {
 		return false;
+	}
 
-	scoped_lock l(m_sync_);
+	fz::scoped_lock l(m_sync_);
 	return m_optionsCache[nID].from_default;
 }
 
 pugi::xml_node COptions::CreateSettingsXmlElement()
 {
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return pugi::xml_node();
+	}
 
 	auto element = m_pXmlFile->GetElement();
 	if (!element) {
@@ -393,35 +403,35 @@ pugi::xml_node COptions::CreateSettingsXmlElement()
 
 void COptions::SetXmlValue(unsigned int nID, int value)
 {
-	SetXmlValue(nID, wxString::Format(_T("%d"), value));
+	SetXmlValue(nID, fz::to_wstring(value));
 }
 
-void COptions::SetXmlValue(unsigned int nID, wxString const& value)
+void COptions::SetXmlValue(unsigned int nID, std::wstring const& value)
 {
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return;
+	}
 
 	// No checks are made about the validity of the value, that's done in SetOption
-
-	wxScopedCharBuffer utf8 = value.utf8_str();
-	if (!utf8)
-		return;
+	std::string utf8 = fz::to_utf8(value);
 
 	auto settings = CreateSettingsXmlElement();
 	if (settings) {
 		pugi::xml_node setting;
 		for (setting = settings.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
 			const char *attribute = setting.attribute("name").value();
-			if (!attribute)
+			if (!attribute) {
 				continue;
-			if (!strcmp(attribute, options[nID].name))
+			}
+			if (!strcmp(attribute, options[nID].name)) {
 				break;
+			}
 		}
 		if (!setting) {
 			setting = settings.append_child("Setting");
 			SetTextAttribute(setting, "name", options[nID].name);
 		}
-		setting.text() = utf8;
+		setting.text() = utf8.c_str();
 	}
 }
 
@@ -430,71 +440,87 @@ int COptions::Validate(unsigned int nID, int value)
 	switch (nID)
 	{
 	case OPTION_UPDATECHECK_INTERVAL:
-		if (value < 1 || value > 7)
+		if (value < 1 || value > 7) {
 			value = 7;
+		}
 		break;
 	case OPTION_LOGGING_DEBUGLEVEL:
-		if (value < 0 || value > 4)
+		if (value < 0 || value > 4) {
 			value = 0;
+		}
 		break;
 	case OPTION_RECONNECTCOUNT:
-		if (value < 0 || value > 99)
+		if (value < 0 || value > 99) {
 			value = 5;
+		}
 		break;
 	case OPTION_RECONNECTDELAY:
-		if (value < 0 || value > 999)
+		if (value < 0 || value > 999) {
 			value = 5;
+		}
 		break;
 	case OPTION_FILEPANE_LAYOUT:
-		if (value < 0 || value > 3)
+		if (value < 0 || value > 3) {
 			value = 0;
+		}
 		break;
 	case OPTION_SPEEDLIMIT_INBOUND:
 	case OPTION_SPEEDLIMIT_OUTBOUND:
-		if (value < 0)
+		if (value < 0) {
 			value = 0;
+		}
 		break;
 	case OPTION_SPEEDLIMIT_BURSTTOLERANCE:
-		if (value < 0 || value > 2)
+		if (value < 0 || value > 2) {
 			value = 0;
+		}
 		break;
 	case OPTION_FILELIST_DIRSORT:
 	case OPTION_FILELIST_NAMESORT:
-		if (value < 0 || value > 2)
+		if (value < 0 || value > 2) {
 			value = 0;
+		}
 		break;
 	case OPTION_SOCKET_BUFFERSIZE_RECV:
-		if (value != -1 && (value < 4096 || value > 4096 * 1024))
+		if (value != -1 && (value < 4096 || value > 4096 * 1024)) {
 			value = -1;
+		}
 		break;
 	case OPTION_SOCKET_BUFFERSIZE_SEND:
-		if (value != -1 && (value < 4096 || value > 4096 * 1024))
+		if (value != -1 && (value < 4096 || value > 4096 * 1024)) {
 			value = 131072;
+		}
 		break;
 	case OPTION_COMPARISONMODE:
-		if (value < 0 || value > 0)
+		if (value < 0 || value > 0) {
 			value = 1;
+		}
 		break;
 	case OPTION_COMPARISON_THRESHOLD:
-		if (value < 0 || value > 1440)
+		if (value < 0 || value > 1440) {
 			value = 1;
+		}
 		break;
 	case OPTION_SIZE_DECIMALPLACES:
-		if (value < 0 || value > 3)
+		if (value < 0 || value > 3) {
 			value = 0;
+		}
 		break;
 	case OPTION_MESSAGELOG_POSITION:
-		if (value < 0 || value > 2)
+		if (value < 0 || value > 2) {
 			value = 0;
+		}
 		break;
 	case OPTION_DOUBLECLICK_ACTION_FILE:
 	case OPTION_DOUBLECLICK_ACTION_DIRECTORY:
-		if (value < 0 || value > 3)
+		if (value < 0 || value > 3) {
 			value = 0;
+		}
 		break;
 	case OPTION_SIZE_FORMAT:
-		if (value < 0 || value >= CSizeFormat::formats_count)
+		if (value < 0 || value >= CSizeFormat::formats_count) {
 			value = 0;
+		}
 		break;
 	case OPTION_TIMEOUT:
 		if (value <= 0) {
@@ -507,86 +533,99 @@ int COptions::Validate(unsigned int nID, int value)
 			value = 9999;
 		}
 		break;
+	case OPTION_CACHE_TTL:
+		if (value < 30) {
+			value = 30;
+		}
+		else if (value > 60*60*24) {
+			value = 60 * 60 * 24;
+		}
+		break;
 	}
 	return value;
 }
 
-wxString COptions::Validate(unsigned int nID, wxString const& value)
+std::wstring COptions::Validate(unsigned int nID, std::wstring const& value)
 {
 	if (nID == OPTION_INVALID_CHAR_REPLACE) {
-		if (value.Len() > 1)
+		if (value.size() > 1) {
 			return _T("_");
+		}
 	}
 	return value;
 }
 
-void COptions::SetServer(wxString path, const CServer& server)
+void COptions::SetServer(std::wstring path, const CServer& server)
 {
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return;
+	}
 
-	if (path.empty())
+	if (path.empty()) {
 		return;
+	}
 
 	auto element = m_pXmlFile->GetElement();
 
 	while (!path.empty()) {
-		wxString sub;
-		int pos = path.Find('/');
-		if (pos != -1) {
-			sub = path.Left(pos);
-			path = path.Mid(pos + 1);
+		std::wstring sub;
+		size_t pos = path.find('/');
+		if (pos != std::wstring::npos) {
+			sub = path.substr(0, pos);
+			path = path.substr(pos + 1);
 		}
 		else {
 			sub = path;
-			path = _T("");
+			path.clear();
 		}
-		wxScopedCharBuffer utf8 = sub.utf8_str();
-		if (!utf8)
-			return;
-		auto newElement = element.child(utf8);
-		if (newElement)
+
+		std::string utf8 = fz::to_utf8(sub);
+		auto newElement = element.child(utf8.c_str());
+		if (newElement) {
 			element = newElement;
+		}
 		else {
-			element = element.append_child(utf8);
+			element = element.append_child(utf8.c_str());
 		}
 	}
 
 	::SetServer(element, server);
 
-	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
+	}
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
 	m_pXmlFile->Save(true);
 }
 
-bool COptions::GetServer(wxString path, CServer& server)
+bool COptions::GetServer(std::wstring path, CServer& server)
 {
-	if (path.empty())
+	if (path.empty()) {
 		return false;
+	}
 
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return false;
+	}
 	auto element = m_pXmlFile->GetElement();
 
 	while (!path.empty()) {
-		wxString sub;
-		int pos = path.Find('/');
-		if (pos != -1) {
-			sub = path.Left(pos);
-			path = path.Mid(pos + 1);
+		std::wstring sub;
+		size_t pos = path.find('/');
+		if (pos != std::wstring::npos) {
+			sub = path.substr(0, pos);
+			path = path.substr(pos + 1);
 		}
 		else {
 			sub = path;
 			path = _T("");
 		}
-		wxScopedCharBuffer utf8 = sub.utf8_str();
-		if (!utf8)
-			return false;
-		element = element.child(utf8);
-		if (!element)
+		std::string utf8 = fz::to_utf8(sub);
+		element = element.child(utf8.c_str());
+		if (!element) {
 			return false;
+		}
 	}
 
 	bool res = ::GetServer(element, server);
@@ -596,10 +635,12 @@ bool COptions::GetServer(wxString path, CServer& server)
 
 void COptions::SetLastServer(const CServer& server)
 {
-	if (!m_pLastServer)
+	if (!m_pLastServer) {
 		m_pLastServer = new CServer(server);
-	else
+	}
+	else {
 		*m_pLastServer = server;
+	}
 	SetServer(_T("Settings/LastServer"), server);
 }
 
@@ -607,14 +648,16 @@ bool COptions::GetLastServer(CServer& server)
 {
 	if (!m_pLastServer) {
 		bool res = GetServer(_T("Settings/LastServer"), server);
-		if (res)
+		if (res) {
 			m_pLastServer = new CServer(server);
+		}
 		return res;
 	}
 	else {
 		server = *m_pLastServer;
-		if (server == CServer())
+		if (server == CServer()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -622,14 +665,16 @@ bool COptions::GetLastServer(CServer& server)
 
 void COptions::Init()
 {
-	if (!m_theOptions)
+	if (!m_theOptions) {
 		new COptions(); // It sets m_theOptions internally itself
+	}
 }
 
 void COptions::Destroy()
 {
-	if (!m_theOptions)
+	if (!m_theOptions) {
 		return;
+	}
 
 	delete m_theOptions;
 	m_theOptions = 0;
@@ -643,8 +688,9 @@ COptions* COptions::Get()
 void COptions::Import(pugi::xml_node element)
 {
 	LoadOptions(GetNameOptionMap(), element);
-	if (!m_save_timer.IsRunning())
+	if (!m_save_timer.IsRunning()) {
 		m_save_timer.Start(15000, true);
+	}
 }
 
 void COptions::LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, pugi::xml_node settings)
@@ -664,36 +710,38 @@ void COptions::LoadOptions(std::map<std::string, unsigned int> const& nameOption
 void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault)
 {
 	const char* name = option.attribute("name").value();
-	if (!name)
+	if (!name) {
 		return;
+	}
 
 	auto const iter = nameOptionMap.find(name);
 	if (iter != nameOptionMap.end()) {
-		if (!allowDefault && options[iter->second].flags == default_only)
+		if (!allowDefault && options[iter->second].flags == default_only) {
 			return;
-		wxString value = GetTextElement(option);
+		}
+		std::wstring value = GetTextElement(option);
 		if (options[iter->second].flags == default_priority) {
 			if (allowDefault) {
-				scoped_lock l(m_sync_);
+				fz::scoped_lock l(m_sync_);
 				m_optionsCache[iter->second].from_default = true;
 			}
 			else {
-				scoped_lock l(m_sync_);
-				if (m_optionsCache[iter->second].from_default)
+				fz::scoped_lock l(m_sync_);
+				if (m_optionsCache[iter->second].from_default) {
 					return;
+				}
 			}
 		}
 
 		if (options[iter->second].type == number) {
-			long numValue = 0;
-			value.ToLong(&numValue);
+			int numValue = fz::to_integral<int>(value);
 			numValue = Validate(iter->second, numValue);
-			scoped_lock l(m_sync_);
+			fz::scoped_lock l(m_sync_);
 			m_optionsCache[iter->second] = numValue;
 		}
 		else {
 			value = Validate(iter->second, value);
-			scoped_lock l(m_sync_);
+			fz::scoped_lock l(m_sync_);
 			m_optionsCache[iter->second] = value;
 		}
 	}
@@ -702,20 +750,23 @@ void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string
 void COptions::LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir.empty())
+	if (defaultsDir.empty()) {
 		return;
-
+	}
 	CXmlFile file(defaultsDir.GetPath() + _T("fzdefaults.xml"));
-	if (!file.Load())
+	if (!file.Load()) {
 		return;
+	}
 
 	auto element = file.GetElement();
-	if (!element)
+	if (!element) {
 		return;
+	}
 
 	element = element.child("Settings");
-	if (!element)
+	if (!element) {
 		return;
+	}
 
 	for (auto setting = element.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
 		LoadOptionFromElement(setting, nameOptionMap, true);
@@ -729,11 +780,13 @@ void COptions::OnTimer(wxTimerEvent&)
 
 void COptions::Save()
 {
-	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
+	}
 
-	if (!m_pXmlFile)
+	if (!m_pXmlFile) {
 		return;
+	}
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
 	m_pXmlFile->Save(true);
@@ -741,26 +794,27 @@ void COptions::Save()
 
 void COptions::SaveIfNeeded()
 {
-	if (!m_save_timer.IsRunning())
+	if (!m_save_timer.IsRunning()) {
 		return;
+	}
 
 	m_save_timer.Stop();
 	Save();
 }
 
 namespace {
-#ifndef __WXMSW__
-wxString TryDirectory( wxString path, wxString const& suffix, bool check_exists )
+#ifndef FZ_WINDOWS
+std::wstring TryDirectory(wxString path, wxString const& suffix, bool check_exists)
 {
-	if( !path.empty() && path[0] == '/' ) {
-		if( path[path.size() - 1] != '/' ) {
+	if (!path.empty() && path[0] == '/') {
+		if (path[path.size() - 1] != '/') {
 			path += '/';
 		}
 
 		path += suffix;
 
-		if( check_exists ) {
-			if( !wxFileName::DirExists(path) ) {
+		if (check_exists) {
+			if (!wxFileName::DirExists(path)) {
 				path.clear();
 			}
 		}
@@ -768,80 +822,106 @@ wxString TryDirectory( wxString path, wxString const& suffix, bool check_exists
 	else {
 		path.clear();
 	}
-	return path;
+	return path.ToStdWstring();
 }
-#endif
 
 wxString GetEnv(wxString const& env)
 {
 	wxString ret;
-	if( !wxGetEnv(env, &ret) ) {
+	if (!wxGetEnv(env, &ret)) {
 		ret.clear();
 	}
 	return ret;
 }
+#endif
 }
 
 CLocalPath COptions::GetUnadjustedSettingsDir()
 {
-	wxFileName fn;
-#ifdef __WXMSW__
-	wxChar buffer[MAX_PATH * 2 + 1];
+	CLocalPath ret;
+
+#ifdef FZ_WINDOWS
+	wchar_t buffer[MAX_PATH * 2 + 1];
 
 	if (SUCCEEDED(SHGetFolderPath(0, CSIDL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer))) {
-		fn = wxFileName(buffer, _T(""));
-		fn.AppendDir(_T("FileZilla"));
+		CLocalPath tmp(buffer);
+		if (!tmp.empty()) {
+			tmp.AddSegment(L"FileZilla");
+		}
+		ret = tmp;
 	}
-	else {
+
+	if (ret.empty()) {
 		// Fall back to directory where the executable is
-		if (GetModuleFileName(0, buffer, MAX_PATH * 2))
-			fn = buffer;
+		DWORD c = GetModuleFileName(0, buffer, MAX_PATH * 2);
+		if (c && c < MAX_PATH * 2) {
+			std::wstring tmp;
+			ret.SetPath(buffer, &tmp);
+		}
 	}
 #else
-	wxString cfg = TryDirectory(GetEnv(_T("XDG_CONFIG_HOME")), _T("filezilla/"), true);
-	if( cfg.empty() ) {
+	std::wstring cfg = TryDirectory(GetEnv(_T("XDG_CONFIG_HOME")), _T("filezilla/"), true);
+	if (cfg.empty()) {
 		cfg = TryDirectory(wxGetHomeDir(), _T(".config/filezilla/"), true);
 	}
-	if( cfg.empty() ) {
+	if (cfg.empty()) {
 		cfg = TryDirectory(wxGetHomeDir(), _T(".filezilla/"), true);
 	}
-	if( cfg.empty() ) {
+	if (cfg.empty()) {
 		cfg = TryDirectory(GetEnv(_T("XDG_CONFIG_HOME")), _T("filezilla/"), false);
 	}
-	if( cfg.empty() ) {
+	if (cfg.empty()) {
 		cfg = TryDirectory(wxGetHomeDir(), _T(".config/filezilla/"), false);
 	}
-	if( cfg.empty() ) {
+	if (cfg.empty()) {
 		cfg = TryDirectory(wxGetHomeDir(), _T(".filezilla/"), false);
 	}
-	fn = wxFileName(cfg, _T(""));
+	ret.SetPath(cfg);
 #endif
-	return CLocalPath(fn.GetPath());
+	return ret;
+}
+
+CLocalPath COptions::GetCacheDirectory()
+{
+	CLocalPath ret;
+
+	std::wstring dir(GetOption(OPTION_DEFAULT_CACHE_DIR));
+	if (!dir.empty()) {
+		dir = ExpandPath(dir);
+		ret.SetPath(wxGetApp().GetDefaultsDir().GetPath());
+		if (!ret.ChangePath(dir)) {
+			ret.clear();
+		}
+	}
+	else {
+#ifdef FZ_WINDOWS
+		wchar_t buffer[MAX_PATH * 2 + 1];
+
+		if (SUCCEEDED(SHGetFolderPath(0, CSIDL_LOCAL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer))) {
+			ret.SetPath(buffer);
+			if (!ret.empty()) {
+				ret.AddSegment(L"FileZilla");
+			}
+		}
+#else
+		std::wstring cfg = TryDirectory(GetEnv(_T("XDG_CACHE_HOME")), _T("filezilla/"), false);
+		if (cfg.empty()) {
+			cfg = TryDirectory(wxGetHomeDir(), _T(".cache/filezilla/"), false);
+		}
+		ret.SetPath(cfg);
+#endif
+	}
+
+	return ret;
 }
 
 CLocalPath COptions::InitSettingsDir()
 {
 	CLocalPath p;
 
-	wxString dir(GetOption(OPTION_DEFAULT_SETTINGSDIR));
+	std::wstring dir = GetOption(OPTION_DEFAULT_SETTINGSDIR);
 	if (!dir.empty()) {
-		wxStringTokenizer tokenizer(dir, _T("/\\"), wxTOKEN_RET_EMPTY_ALL);
-		dir = _T("");
-		while (tokenizer.HasMoreTokens()) {
-			wxString token = tokenizer.GetNextToken();
-			if (!token.empty() && token[0] == '$') {
-				if (token.size() > 1 && token[1] == '$')
-					token = token.Mid(1);
-				else {
-					token = GetEnv(token.Mid(1));
-				}
-			}
-			dir += token;
-			const wxChar delimiter = tokenizer.GetLastDelimiter();
-			if (delimiter)
-				dir += delimiter;
-		}
-
+		dir = ExpandPath(dir);
 		p.SetPath(wxGetApp().GetDefaultsDir().GetPath());
 		p.ChangePath(dir);
 	}
@@ -849,8 +929,9 @@ CLocalPath COptions::InitSettingsDir()
 		p = GetUnadjustedSettingsDir();
 	}
 
-	if (!p.empty() && !p.Exists())
-		wxFileName::Mkdir( p.GetPath(), 0700, wxPATH_MKDIR_FULL );
+	if (!p.empty() && !p.Exists()) {
+		wxFileName::Mkdir(p.GetPath(), 0700, wxPATH_MKDIR_FULL);
+	}
 
 	SetOption(OPTION_DEFAULT_SETTINGSDIR, p.GetPath());
 
@@ -859,7 +940,7 @@ CLocalPath COptions::InitSettingsDir()
 
 void COptions::SetDefaultValues()
 {
-	scoped_lock l(m_sync_);
+	fz::scoped_lock l(m_sync_);
 	for (int i = 0; i < OPTIONS_NUM; ++i) {
 		m_optionsCache[i] = options[i].defaultValue;
 		m_optionsCache[i].from_default = false;
