@@ -4,12 +4,13 @@
 #include "edithandler.h"
 #include "filezillaapp.h"
 #include "file_utils.h"
-#include "local_filesys.h"
 #include "Options.h"
 #include "queue.h"
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 class CChangedFileDialog : public wxDialogEx
 {
 	DECLARE_EVENT_TABLE()
@@ -101,7 +102,7 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 		if (fd >= 0)
 		{
 			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-			struct flock f = { 0 };
+			struct flock f = {};
 			f.l_type = F_WRLCK;
 			f.l_whence = SEEK_SET;
 			f.l_start = 0;
@@ -138,10 +139,11 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 
 }
 
-wxString CEditHandler::GetLocalDirectory()
+std::wstring CEditHandler::GetLocalDirectory()
 {
-	if (!m_localDir.empty())
+	if (!m_localDir.empty()) {
 		return m_localDir;
+	}
 
 	wxFileName tmpdir(wxFileName::GetTempDir(), _T(""));
 	// Need to call GetLongPath on MSW, GetTempDir can return short path
@@ -149,7 +151,7 @@ wxString CEditHandler::GetLocalDirectory()
 	// length
 	wxString dir = tmpdir.GetLongPath();
 	if (dir.empty() || !wxFileName::DirExists(dir))
-		return wxString();
+		return std::wstring();
 
 	if (dir.Last() != wxFileName::GetPathSeparator())
 		dir += wxFileName::GetPathSeparator();
@@ -160,14 +162,16 @@ wxString CEditHandler::GetLocalDirectory()
 	// already has the correct permissions which get inherited.
 	int i = 1;
 	do {
-		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), i++);
-		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir))
+		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), ++i);
+		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir)) {
 			continue;
+		}
 
-		if (!wxMkdir(newDir, 0700))
-			return wxString();
+		if (!wxMkdir(newDir, 0700)) {
+			return std::wstring();
+		}
 
-		m_localDir = newDir + wxFileName::GetPathSeparator();
+		m_localDir = (newDir + wxFileName::GetPathSeparator()).ToStdWstring();
 		break;
 	} while (true);
 
@@ -178,19 +182,17 @@ wxString CEditHandler::GetLocalDirectory()
 	RemoveTemporaryFiles(dir);
 
 #ifdef __WXMSW__
-	m_lockfile_handle = ::CreateFile(m_localDir + _T("fz3temp-lockfile"), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
-	if (m_lockfile_handle == INVALID_HANDLE_VALUE)
-	{
+	m_lockfile_handle = ::CreateFile((m_localDir + L"fz3temp-lockfile").c_str(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
+	if (m_lockfile_handle == INVALID_HANDLE_VALUE) {
 		wxRmdir(m_localDir);
-		m_localDir = _T("");
+		m_localDir.clear();
 	}
 #else
-	wxString file = m_localDir + _T("fz3temp-lockfile");
-	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
-	if (m_lockfile_descriptor >= 0)
-	{
+	auto file = fz::to_native(m_localDir) + "fz3temp-lockfile";
+	m_lockfile_descriptor = open(file.c_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
+	if (m_lockfile_descriptor >= 0) {
 		// Lock 1 byte region in the lockfile.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = 0;
@@ -231,7 +233,7 @@ void CEditHandler::Release()
 	delete this;
 }
 
-enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName) const
+CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
@@ -240,7 +242,7 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 	return iter->state;
 }
 
-enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
+CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
 {
 	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
@@ -249,7 +251,7 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 	return iter->state;
 }
 
-int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandler::fileState state, const CServer* pServer /*=0*/) const
+int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, const CServer* pServer /*=0*/) const
 {
 	int count = 0;
 	if (state == unknown) {
@@ -261,15 +263,17 @@ int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandl
 	}
 	else {
 		auto f = [state, pServer](decltype(m_fileDataList[0]) & items) {
-			int count = 0;
+			int cnt = 0;
 			for (auto const& data : items) {
-				if (data.state != state)
+				if (data.state != state) {
 					continue;
+				}
 
-				if (!pServer || data.server == *pServer)
-					++count;
+				if (!pServer || data.server == *pServer) {
+					++cnt;
+				}
 			}
-			return count;
+			return cnt;
 		};
 		if (type != remote) {
 			count += f(m_fileDataList[local]);
@@ -282,15 +286,17 @@ int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandl
 	return count;
 }
 
-bool CEditHandler::AddFile(enum CEditHandler::fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server)
+bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, CServer const& server)
 {
 	wxASSERT(type != none);
 
 	fileState state;
-	if (type == local)
+	if (type == local) {
 		state = GetFileState(fileName);
-	else
+	}
+	else {
 		state = GetFileState(fileName, remotePath, server);
+	}
 	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
 		return false;
@@ -394,7 +400,7 @@ bool CEditHandler::RemoveAll(bool force)
 	return m_fileDataList[local].empty() && m_fileDataList[remote].empty();
 }
 
-bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/)
+bool CEditHandler::RemoveAll(fileState state, const CServer* pServer /*=0*/)
 {
 	// Others not implemented
 	wxASSERT(state == upload_and_remove_failed);
@@ -487,7 +493,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
+void CEditHandler::FinishTransfer(bool, const wxString& fileName)
 {
 	auto iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
@@ -579,13 +585,13 @@ bool CEditHandler::StartEditing(const wxString& file, const CServerPath& remoteP
 	return StartEditing(remote, *iter);
 }
 
-bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& data)
+bool CEditHandler::StartEditing(CEditHandler::fileType type, t_fileData& data)
 {
 	wxASSERT(type != none);
 	wxASSERT(data.state == edit);
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(data.file, is_link, 0, &data.modificationTime, 0) != CLocalFileSystem::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file)
 		return false;
 
 	bool program_exists = false;
@@ -618,19 +624,19 @@ checkmodifications_loopbegin:
 			if (iter->state != edit)
 				continue;
 
-			CDateTime mtime;
+			fz::datetime mtime;
 			bool is_link;
-			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
+			if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 
 				// Evil goto. Imo the next C++ standard needs a comefrom keyword.
 				goto checkmodifications_loopbegin;
 			}
 
-			if (!mtime.IsValid())
+			if (mtime.empty())
 				continue;
 
-			if (iter->modificationTime.IsValid() && !iter->modificationTime.Compare(mtime))
+			if (!iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
@@ -659,7 +665,7 @@ checkmodifications_loopbegin:
 			}
 			else if (remove) {
 				if (i == static_cast<int>(remote)) {
-					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file)) {
+					if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file || wxRemoveFile(iter->file)) {
 						m_fileDataList[i].erase(iter);
 						goto checkmodifications_loopbegin;
 					}
@@ -670,7 +676,7 @@ checkmodifications_loopbegin:
 					goto checkmodifications_loopbegin;
 				}
 			}
-			else if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
+			else if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
 			}
@@ -715,7 +721,8 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 		else
 			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
 	}
-	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
+
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.Format(ServerFormat::with_user_and_optional_port));
 	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
@@ -739,7 +746,7 @@ bool CEditHandler::UploadFile(const wxString& file, bool unedit)
 	return UploadFile(local, iter, unedit);
 }
 
-bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterator iter, bool unedit)
+bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
 	wxCHECK(type != none, false);
 
@@ -753,26 +760,25 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	iter->state = unedit ? upload_and_remove : upload;
 
 	int64_t size;
-	CDateTime mtime;
+	fz::datetime mtime;
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(iter->file, is_link, &size, &mtime, 0) != CLocalFileSystem::file)
-	{
+	if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, &size, &mtime, 0) != fz::local_filesys::file) {
 		m_fileDataList[type].erase(iter);
 		return false;
 	}
 
-	if (!mtime.IsValid())
-		mtime = CDateTime::Now();
+	if (mtime.empty()) {
+		mtime = fz::datetime::now();
+	}
 
 	iter->modificationTime = mtime;
 
 	wxASSERT(m_pQueue);
 
-	wxString file;
+	std::wstring file;
 	CLocalPath localPath(iter->file, &file);
-	if (file.empty())
-	{
+	if (file.empty()) {
 		m_fileDataList[type].erase(iter);
 		return false;
 	}
@@ -808,7 +814,7 @@ void CEditHandler::SetTimerState()
 		m_timer.Start(15000);
 }
 
-wxString CEditHandler::CanOpen(enum CEditHandler::fileType type, const wxString& fileName, bool &dangerous, bool &program_exists)
+wxString CEditHandler::CanOpen(CEditHandler::fileType type, const wxString& fileName, bool &dangerous, bool &program_exists)
 {
 	wxASSERT(type != none);
 
@@ -939,8 +945,9 @@ void CEditHandler::OnChangedFileEvent(wxCommandEvent&)
 	CheckForModifications();
 }
 
-wxString CEditHandler::GetTemporaryFile(wxString name)
+std::wstring CEditHandler::GetTemporaryFile(std::wstring name)
 {
+	name = CQueueView::ReplaceInvalidCharacters(name);
 #ifdef __WXMSW__
 	// MAX_PATH - 1 is theoretical limit, we subtract another 4 to allow
 	// editors which create temporary files
@@ -948,44 +955,48 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 #else
 	int max = -1;
 #endif
-	if (max != -1)
-	{
-		name = TruncateFilename(m_localDir, name, max);
-		if (name.empty())
-			return wxString();
+	if (max != -1) {
+		name = TruncateFilename(m_localDir, name, max).ToStdWstring();
+		if (name.empty()) {
+			return std::wstring();
+		}
 	}
 
-	wxString file = m_localDir + name;
-	if (!FilenameExists(file))
+	std::wstring file = m_localDir + name;
+	if (!FilenameExists(file)) {
 		return file;
+	}
 
-	if (max != -1)
+	if (max != -1) {
 		max--;
+	}
 	int cutoff = 1;
 	int n = 1;
-	while (++n < 10000) // Just to give up eventually
-	{
+	while (++n < 10000) { // Just to give up eventually
 		// Further reduce length if needed
-		if (max != -1 && n >= cutoff)
-		{
+		if (max != -1 && n >= cutoff) {
 			cutoff *= 10;
 			max--;
 			name = TruncateFilename(m_localDir, name, max);
-			if (name.empty())
-				return wxString();
+			if (name.empty()) {
+				return std::wstring();
+			}
 		}
 
-		int pos = name.Find('.', true);
-		if (pos < 1)
-			file = m_localDir + name + wxString::Format(_T(" %d"), n);
-		else
-			file = m_localDir + name.Left(pos) + wxString::Format(_T(" %d"), n) + name.Mid(pos);
+		size_t pos = name.rfind('.');
+		if (pos == std::wstring::npos || !pos) {
+			file = m_localDir + name + fz::sprintf(L" %d", n);
+		}
+		else {
+			file = m_localDir + name.substr(0, pos) + fz::sprintf(L" %d", n) + name.substr(pos);
+		}
 
-		if (!FilenameExists(file))
+		if (!FilenameExists(file)) {
 			return file;
+		}
 	}
 
-	return wxString();
+	return std::wstring();
 }
 
 wxString CEditHandler::TruncateFilename(const wxString path, const wxString& name, int max)
@@ -1016,8 +1027,9 @@ bool CEditHandler::FilenameExists(const wxString& file)
 	for (auto const& fileData : m_fileDataList[remote]) {
 		// Always ignore case, we don't know which type of filesystem the user profile
 		// is installed upon.
-		if (!fileData.file.CmpNoCase(file))
+		if (!wxString(fileData.file).CmpNoCase(file)) {
 			return true;
+		}
 	}
 
 	if (wxFileName::FileExists(file)) {
@@ -1117,7 +1129,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.FormatServer() + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1145,7 +1157,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.FormatServer() + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1184,7 +1196,7 @@ void CEditHandlerStatusDialog::SetCtrlState()
 	int item = -1;
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 		if (pData->state == CEditHandler::edit)
 			selectedEdited = true;
@@ -1218,7 +1230,7 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 	int item = -1;
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
@@ -1231,7 +1243,7 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		if (type == CEditHandler::local) {
@@ -1265,7 +1277,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
 		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
@@ -1278,7 +1290,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		bool unedit = event.GetId() == XRCID("ID_UPLOADANDUNEDIT") || pData->state == CEditHandler::upload_and_remove_failed;
@@ -1312,7 +1324,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
 		if (pData->state != CEditHandler::edit) {
@@ -1325,7 +1337,7 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
-		enum CEditHandler::fileType type;
+		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
 		if (type == CEditHandler::local) {
@@ -1442,7 +1454,7 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 void CNewAssociationDialog::SetCtrlState()
 {
-	wxRadioButton* pCustom = wxDynamicCast(FindWindow(XRCID("ID_USE_CUSTOM")), wxRadioButton);
+	wxRadioButton* pCustom = dynamic_cast<wxRadioButton*>(FindWindow(XRCID("ID_USE_CUSTOM")));
 	if (!pCustom) {
 		// Return since it can get called before dialog got fully loaded
 		return;
@@ -1478,15 +1490,15 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 		}
 
 		if (always)
-			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd);
+			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd.ToStdWstring());
 		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
 			if (!associations.empty() && associations.Last() != '\n')
 				associations += '\n';
 			if (m_ext.empty())
 				m_ext = _T("/");
-			associations += m_ext + _T(" ") + cmd;
-			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
+			associations += m_ext.ToStdWstring() + _T(" ") + cmd.ToStdWstring();
+			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 	else {
@@ -1519,7 +1531,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 			if (m_ext.empty())
 				m_ext = _T("/");
 			associations += m_ext + _T(" ") + cmd;
-			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
+			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 
@@ -1557,7 +1569,7 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, int64_t size, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, CServer const& server, int64_t size, wxWindow * parent)
 {
 	std::vector<FileData> data;
 	FileData d{fileName, size};
@@ -1727,7 +1739,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		break;
 	}
 
-	wxString localFile = file.name;
+	std::wstring localFile = file.name;
 	if (!AddFile(type, localFile, path, server)) {
 		if( type == CEditHandler::local) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
@@ -1740,10 +1752,10 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	if (type == CEditHandler::remote) {
-		wxString localFileName;
+		std::wstring localFileName;
 		CLocalPath localPath(localFile, &localFileName);
 
-		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : wxString(),
+		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : std::wstring(),
 			localPath, path, server, file.size, type, QueuePriority::high);
 		m_pQueue->QueueFile_Finish(true);
 	}
