@@ -1,6 +1,5 @@
 #include <filezilla.h>
 #include "manual_transfer.h"
-#include "local_filesys.h"
 #include "auto_ascii_files.h"
 #include "state.h"
 #include "Options.h"
@@ -9,6 +8,8 @@
 #include "QueueView.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 BEGIN_EVENT_TABLE(CManualTransfer, wxDialogEx)
 EVT_TEXT(XRCID("ID_LOCALFILE"), CManualTransfer::OnLocalChanged)
 EVT_TEXT(XRCID("ID_REMOTEFILE"), CManualTransfer::OnRemoteChanged)
@@ -58,7 +59,7 @@ void CManualTransfer::Run(wxWindow* pParent, CState* pState)
 	wxChoice* pChoice = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice);
 	wxASSERT(pChoice);
 	for (int i = 0; i < LOGONTYPE_MAX; ++i)
-		pChoice->Append(CServer::GetNameFromLogonType((enum LogonType)i));
+		pChoice->Append(CServer::GetNameFromLogonType(static_cast<LogonType>(i)));
 
 	if (m_pState->GetServer()) {
 		m_pServer = new CServer(*m_pState->GetServer());
@@ -158,7 +159,7 @@ void CManualTransfer::DisplayServer()
 {
 	if (m_pServer)
 	{
-		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(m_pServer->FormatHost(true));
+		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(m_pServer->Format(ServerFormat::host_only));
 		unsigned int port = m_pServer->GetPort();
 
 		if (port != CServer::GetDefaultPort(m_pServer->GetProtocol()))
@@ -211,24 +212,24 @@ void CManualTransfer::DisplayServer()
 	}
 }
 
-void CManualTransfer::OnLocalChanged(wxCommandEvent& event)
+void CManualTransfer::OnLocalChanged(wxCommandEvent&)
 {
 	if (XRCCTRL(*this, "ID_DOWNLOAD", wxRadioButton)->GetValue())
 		return;
 
 	wxString file = XRCCTRL(*this, "ID_LOCALFILE", wxTextCtrl)->GetValue();
 
-	m_local_file_exists = CLocalFileSystem::GetFileType(file) == CLocalFileSystem::file;
+	m_local_file_exists = fz::local_filesys::get_file_type(fz::to_native(file)) == fz::local_filesys::file;
 
 	SetAutoAsciiState();
 }
 
-void CManualTransfer::OnRemoteChanged(wxCommandEvent& event)
+void CManualTransfer::OnRemoteChanged(wxCommandEvent&)
 {
 	SetAutoAsciiState();
 }
 
-void CManualTransfer::OnLocalBrowse(wxCommandEvent& event)
+void CManualTransfer::OnLocalBrowse(wxCommandEvent&)
 {
 	int flags;
 	wxString title;
@@ -283,10 +284,11 @@ void CManualTransfer::OnServerTypeChanged(wxCommandEvent& event)
 	SetServerState();
 }
 
-void CManualTransfer::OnOK(wxCommandEvent& event)
+void CManualTransfer::OnOK(wxCommandEvent&)
 {
-	if (!UpdateServer())
+	if (!UpdateServer()) {
 		return;
+	}
 
 	bool download = xrc_call(*this, "ID_DOWNLOAD", &wxRadioButton::GetValue);
 
@@ -297,30 +299,30 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 		return;
 	}
 
-	wxString local_file = xrc_call(*this, "ID_LOCALFILE", &wxTextCtrl::GetValue);
+	std::wstring local_file = xrc_call(*this, "ID_LOCALFILE", &wxTextCtrl::GetValue).ToStdWstring();
 	if (local_file.empty()) {
 		wxMessageBoxEx(_("You need to specify a local file."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileType(local_file);
-	if (type == CLocalFileSystem::dir) {
+	fz::local_filesys::type type = fz::local_filesys::get_file_type(fz::to_native(local_file));
+	if (type == fz::local_filesys::dir) {
 		wxMessageBoxEx(_("Local file is a directory instead of a regular file."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
-	if (!download && type != CLocalFileSystem::file && start) {
+	if (!download && type != fz::local_filesys::file && start) {
 		wxMessageBoxEx(_("Local file does not exist."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString remote_file = xrc_call(*this, "ID_REMOTEFILE", &wxTextCtrl::GetValue);
+	std::wstring remote_file = xrc_call(*this, "ID_REMOTEFILE", &wxTextCtrl::GetValue).ToStdWstring();
 
 	if (remote_file.empty()) {
 		wxMessageBoxEx(_("You need to specify a remote file."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString remote_path_str = xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::GetValue);
+	std::wstring remote_path_str = xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::GetValue).ToStdWstring();
 	if (remote_path_str.empty()) {
 		wxMessageBoxEx(_("You need to specify a remote path."), _("Manual transfer"), wxICON_EXCLAMATION);
 		return;
@@ -335,14 +337,17 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 	int old_data_type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
 
 	// Set data type for the file to add
-	if (xrc_call(*this, "ID_TYPE_ASCII", &wxRadioButton::GetValue))
+	if (xrc_call(*this, "ID_TYPE_ASCII", &wxRadioButton::GetValue)) {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 1);
-	else if (xrc_call(*this, "ID_TYPE_BINARY", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_TYPE_BINARY", &wxRadioButton::GetValue)) {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 2);
-	else
+	}
+	else {
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 0);
+	}
 
-	wxString name;
+	std::wstring name;
 	CLocalPath localPath(local_file, &name);
 
 	if (name.empty()) {
@@ -352,7 +357,7 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 
 	m_pQueueView->QueueFile(!start, download,
 		download ? remote_file : name,
-		(remote_file != name) ? (download ? name : remote_file) : wxString(),
+		(remote_file != name) ? (download ? name : remote_file) : std::wstring(),
 		localPath, path, *m_pServer, -1);
 
 	// Restore old data type
@@ -378,27 +383,28 @@ bool CManualTransfer::UpdateServer()
 		return false;
 	}
 
-	wxString host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue);
+	std::wstring host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
 	// SetHost does not accept URL syntax
 	if (!host.empty() && host[0] == '[') {
-		host.RemoveLast();
-		host = host.Mid(1);
+		host = host.substr(1, host.size() - 2);
 	}
 	server.SetHost(host, port);
 
-	const wxString& protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection);
-	const enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
-	if (protocol != UNKNOWN)
+	std::wstring const protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection).ToStdWstring();
+	ServerProtocol const protocol = CServer::GetProtocolFromName(protocolName);
+	if (protocol != UNKNOWN) {
 		server.SetProtocol(protocol);
-	else
+	}
+	else {
 		server.SetProtocol(FTP);
+	}
 
-	enum LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
+	LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection).ToStdWstring());
 	server.SetLogonType(logon_type);
 
-	server.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue),
-		xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue));
-	server.SetAccount(xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue));
+	server.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue).ToStdWstring(),
+		xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue).ToStdWstring());
+	server.SetAccount(xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue).ToStdWstring());
 
 	delete m_pServer;
 	m_pServer = new CServer(server);
@@ -408,17 +414,17 @@ bool CManualTransfer::UpdateServer()
 
 bool CManualTransfer::VerifyServer()
 {
-	const wxString& host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue);
+	std::wstring const host = xrc_call(*this, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
 	if (host.empty()) {
 		xrc_call(*this, "ID_HOST", &wxTextCtrl::SetFocus);
 		wxMessageBoxEx(_("You have to enter a hostname."));
 		return false;
 	}
 
-	enum LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection));
+	LogonType logon_type = CServer::GetLogonTypeFromName(xrc_call(*this, "ID_LOGONTYPE", &wxChoice::GetStringSelection).ToStdWstring());
 
-	wxString protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection);
-	enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
+	std::wstring protocolName = xrc_call(*this, "ID_PROTOCOL", &wxChoice::GetStringSelection).ToStdWstring();
+	ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
 	if (protocol == SFTP &&
 		logon_type == ACCOUNT)
 	{
@@ -453,19 +459,20 @@ bool CManualTransfer::VerifyServer()
 		server.SetProtocol(protocol);
 
 	CServerPath path;
-	wxString error;
-	if (!server.ParseUrl(host, xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue), wxString(), wxString(), error, path)) {
+	std::wstring error;
+	if (!server.ParseUrl(host, xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue).ToStdWstring(), std::wstring(), std::wstring(), error, path)) {
 		xrc_call(*this, "ID_HOST", &wxTextCtrl::SetFocus);
 		wxMessageBoxEx(error);
 		return false;
 	}
 
-	xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, server.FormatHost(true));
+	xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, server.Format(ServerFormat::host_only));
 	xrc_call(*this, "ID_PORT", &wxTextCtrl::ChangeValue, wxString::Format(_T("%d"), server.GetPort()));
 
 	protocolName = CServer::GetProtocolName(server.GetProtocol());
-	if (protocolName.empty())
+	if (protocolName.empty()) {
 		CServer::GetProtocolName(FTP);
+	}
 	xrc_call(*this, "ID_PROTOCOL", &wxChoice::SetStringSelection, protocolName);
 
 	// Require username for non-anonymous, non-ask logon type
@@ -509,7 +516,7 @@ bool CManualTransfer::VerifyServer()
 	return true;
 }
 
-void CManualTransfer::OnSelectSite(wxCommandEvent& event)
+void CManualTransfer::OnSelectSite(wxCommandEvent&)
 {
 	std::unique_ptr<wxMenu> pMenu = CSiteManager::GetSitesMenu();
 	if (pMenu) {
@@ -519,7 +526,7 @@ void CManualTransfer::OnSelectSite(wxCommandEvent& event)
 
 void CManualTransfer::OnSelectedSite(wxCommandEvent& event)
 {
-	std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteById(event.GetId());
+	std::unique_ptr<Site> pData = CSiteManager::GetSiteById(event.GetId());
 	if (!pData)
 		return;
 
