@@ -9,6 +9,7 @@
 #define SECURITY_WIN32
 #endif
 #include <security.h>
+#include <wchar.h>
 
 OSVERSIONINFO osVersion;
 
@@ -101,6 +102,11 @@ char *get_username(void)
 	if (!tried_usernameex) {
 	    /* Not available on Win9x, so load dynamically */
 	    HMODULE secur32 = load_system32_dll("secur32.dll");
+	    /* If MIT Kerberos is installed, the following call to
+	       GET_WINDOWS_FUNCTION makes Windows implicitly load
+	       sspicli.dll WITHOUT proper path sanitizing, so better
+	       load it properly before */
+	    HMODULE sspicli = load_system32_dll("sspicli.dll");
 	    GET_WINDOWS_FUNCTION(secur32, GetUserNameExA);
 	    tried_usernameex = TRUE;
 	}
@@ -149,6 +155,47 @@ char *get_username(void)
     return got_username ? user : NULL;
 }
 
+void dll_hijacking_protection(void)
+{
+    /*
+     * If the OS provides it, call SetDefaultDllDirectories() to
+     * prevent DLLs from being loaded from the directory containing
+     * our own binary, and instead only load from system32.
+     *
+     * This is a protection against hijacking attacks, if someone runs
+     * PuTTY directly from their web browser's download directory
+     * having previously been enticed into clicking on an unwise link
+     * that downloaded a malicious DLL to the same directory under one
+     * of various magic names that seem to be things that standard
+     * Windows DLLs delegate to.
+     *
+     * It shouldn't break deliberate loading of user-provided DLLs
+     * such as GSSAPI providers, because those are specified by their
+     * full pathname by the user-provided configuration.
+     */
+    static HMODULE kernel32_module;
+    DECL_WINDOWS_FUNCTION(static, BOOL, SetDefaultDllDirectories, (DWORD));
+
+    if (!kernel32_module) {
+        kernel32_module = load_system32_dll("kernel32.dll");
+#if defined _MSC_VER && _MSC_VER < 1900
+        /* For older Visual Studio, this function isn't available in
+         * the header files to type-check */
+        GET_WINDOWS_FUNCTION_NO_TYPECHECK(
+            kernel32_module, SetDefaultDllDirectories);
+#else
+        GET_WINDOWS_FUNCTION(kernel32_module, SetDefaultDllDirectories);
+#endif
+    }
+
+    if (p_SetDefaultDllDirectories) {
+        /* LOAD_LIBRARY_SEARCH_SYSTEM32 and explicitly specified
+         * directories only */
+        p_SetDefaultDllDirectories(LOAD_LIBRARY_SEARCH_SYSTEM32 |
+                                   LOAD_LIBRARY_SEARCH_USER_DIRS);
+    }
+}
+
 BOOL init_winver(void)
 {
     ZeroMemory(&osVersion, sizeof(osVersion));
@@ -538,3 +585,76 @@ FontSpec *fontspec_deserialise(void *vdata, int maxsize, int *used)
                         GET_32BIT_MSB_FIRST(end + 4),
                         GET_32BIT_MSB_FIRST(end + 8));
 }
+
+
+FILE *f_open(const struct Filename *filename, char const *mode, int is_private)
+{
+    FILE* f;
+    wchar_t *wname, *wmode;
+
+    (void)is_private;
+
+    if (!filename || !filename->path || !mode) {
+	return NULL;
+    }
+
+    // The filename is in UTF-8, not expressible in system char, need to use wchar_t. Thus we need to use CreateFileW and convert the HANDLE to a FILE*
+    wname = utf8_to_wide(filename->path);
+    wmode = utf8_to_wide(mode);
+    if (!wname || !wmode) {
+	sfree(wname);
+	sfree(wmode);
+	return NULL;
+    }
+
+    f = _wfopen(wname, wmode);
+
+    sfree(wname);
+    sfree(wmode);
+
+    return f;
+}
+
+wchar_t* utf8_to_wide(const char* utf8)
+{
+    wchar_t *w;
+
+    int len = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, 0, 0);
+    if (len <= 0)
+	return NULL;
+
+    w = snewn(len, wchar_t);
+
+    if (!w)
+	return NULL;
+
+    if (MultiByteToWideChar(CP_UTF8, 0, utf8, -1, w, len) <= 0)
+    {
+	sfree(w);
+	return NULL;
+    }
+
+    return w;
+}
+
+char* wide_to_utf8(const wchar_t* w)
+{
+    char* utf8;
+
+    int len = WideCharToMultiByte(CP_UTF8, 0, w, -1, 0, 0, 0, 0);
+    if (len <= 0)
+	return NULL;
+
+    utf8 = snewn(len, char);
+
+    if (!utf8)
+	return NULL;
+
+    if (WideCharToMultiByte(CP_UTF8, 0, w, -1, utf8, len, 0, 0) <= 0)
+    {
+	sfree(utf8);
+	return NULL;
+    }
+    
+    return utf8;
+}
